@import '~styles/variables';

.wrapper {
  min-height: calc(100vh - 108px);
  margin-top: 16px;
  padding: 0 @body-padding;
}

.hint {
  margin-bottom: 16px;
}

.hints {
  :global {
    .ant-alert-warning {
      background-color: #fffbe6;
      border: 1px solid #ffe58f;

      .ant-alert-icon {
        color: @warn-color;
      }
    }

    .ant-alert-message {
      max-height: 40px;
      overflow-y: auto;
    }
  }
}
