@import '~styles/variables';

.confirm-modal {
  :global {
    .ant-modal-confirm-body > .anticon {
      margin-right: 20px;
      font-size: 18px;
    }
  }
}

.confirm {
  color: @warn-color !important;
}

.error {
  color: @error-color !important;
}

.success {
  color: @success-color !important;
}

.warn {
  color: @warn-color !important;
}
