// Copyright 2022 99cloud
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import { imageStatus, imageFormats } from 'resources/glance/image';

export const getBaseSnapshotColumns = (self) => [
  {
    title: t('ID/Name'),
    dataIndex: 'name',
    routeName: self.getRouteName('instanceSnapshotDetail'),
  },
  {
    title: t('Project ID/Name'),
    dataIndex: 'project_name',
    isHideable: true,
    hidden: !self.isAdminPage,
    sorter: false,
  },
  {
    title: t('Description'),
    dataIndex: 'description',
    isHideable: true,
    sorter: false,
  },
  {
    title: t('Disk Format'),
    dataIndex: 'disk_format',
    isHideable: true,
    valueMap: imageFormats,
  },
  {
    title: t('Status'),
    dataIndex: 'status',
    valueMap: imageStatus,
  },
  {
    title: t('Created At'),
    dataIndex: 'created_at',
    isHideable: true,
    valueRender: 'sinceTime',
  },
];
