.container {
  position: relative;
  display: flex;
  flex-flow: row;
  height: 100vh;
  overflow: hidden;
  background-color: #fff;

  .left {
    position: relative;
    width: 576px;
    overflow-y: auto;

    /* TODO wait for RGB
    // background: rgb(253, 249, 252) no-repeat fixed left bottom;
    // background-size: 556px; */

    .lang {
      position: absolute;
      top: 20px;
      right: 24px;
      width: 20px;
      height: 20px;
      text-align: right;
    }

    .main {
      margin-top: 10vh;
      padding: 0 130px;

      .top {
        text-align: center;

        .header {
          height: 56px;
          line-height: 56px;
          text-align: left;

          a {
            text-decoration: none;
          }

          .logo {
            //width: 87px;
            height: 56px;
            margin-right: 16px;
            vertical-align: top;
          }
        }
      }
    }
  }

  .right {
    position: relative;
    flex: 1 1;
    text-align: center;
    background-color: #21242a;
  }
}

.login-image {
  margin-top: calc(100vh - 668px);
}

.login-full-image {
  width: 100%;
  height: 100%;
}

.full-image-front {
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
  left: 0;
  background-color: rgba(14, 34, 181, 15%);
}

.login-right-logo {
  position: relative;
  top: -100px;
}
