// Copyright 2021 99cloud
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import CreateFolder from './CreateFolder';
import Delete from './Delete';
import UploadFile from './UploadFile';
import Download from './Download';
import CopyFile from './CopyFile';
import CutFile from './CutFile';
import Edit from './Edit';
import PasteFile from './PasteFile';
import Rename from './Rename';

const actionConfigs = {
  rowActions: {
    firstAction: Delete,
    moreActions: [
      {
        action: Edit,
      },
      {
        action: Download,
      },
      {
        action: CopyFile,
      },
      {
        action: CutFile,
      },
      {
        action: PasteFile,
      },
      {
        action: Rename,
      },
    ],
  },
  batchActions: [Delete, CopyFile, CutFile],
  primaryActions: [CreateFolder, UploadFile, PasteFile],
};

export default actionConfigs;
