@import '~styles/variables';

@min-space: 8px;
@mid-space: 16px;
@lg-space: 24px;

.main {
  display: flex;
  padding: 0 @mid-space;

  .left-side {
    width: 380px;
    padding-right: @mid-space;
  }

  .right-side {
    flex: 1;
  }
}
