.notify {
  :global {
    .ant-notification-notice-icon {
      font-size: 18px;
    }

    .ant-notification-notice-message {
      font-size: 12px;
      word-break: break-all;
    }

    .ant-notification-notice-with-icon .ant-notification-notice-message {
      margin-left: 32px;
    }
  }
}

.code-editor {
  height: 400px !important;
  min-height: 400px !important;
}
