// Copyright 2021 99cloud
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import { ConfirmAction } from 'containers/Action';
import _ from 'lodash';
import globalFloatingIpsStore from 'stores/neutron/floatingIp';

export default class Release extends ConfirmAction {
  get id() {
    return 'disassociate';
  }

  get title() {
    return t('Disassociate');
  }

  get buttonText() {
    return t('Disassociate');
  }

  get actionName() {
    return t('Disassociate');
  }

  policy = 'update_floatingip';

  allowedCheckFunc = (item) =>
    !_.isNull(item.fixed_ip_address) && item.status !== 'DOWN';

  getItemName = (data) => data.floating_ip_address;

  onSubmit = (value) => globalFloatingIpsStore.disassociateFip(value);
}
