// @import '~antd/es/style/themes/default.less';

// .container > * {
//   background-color: #fff;
//   border-radius: @border-radius;
//   box-shadow: @shadow-1-down;
// }

// @media screen and (max-width: @screen-xs) {
//   .container {
//     width: 100% !important;
//   }
//   .container > * {
//     border-radius: 0 !important;
//   }
// }
