// Copyright 2021 99cloud
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

module.exports = {
  'primary-color': '#0C63FA',
  'link-color': '#0C63FA',
  'success-color': '#52C41A',
  'warning-color': '#FAAD14',
  'error-color': '#D40000',
  'btn-default-color': '#0C63FA',
  'btn-default-border': '#0C63FA',
  'border-color-base': '#E8E8EA',
  'border-radius-base': '2px',
  'font-size-base': '12px',
};
