module.exports = {
  SIMPLE: '简单',
  HELLO: '你好, {name}',
  TIP: '这是<span>HTML</span>',
  TIP_VAR: '这是<span>{message}</span>',
  SALE_START: '拍卖将在{start, date}开始',
  SALE_END: '拍卖将在{start, date, full}开始',
  COUPON: '优惠卷将在{expires, time, medium}过期',
  TIME: '时间是{theTime, time}',
  SALE_PRICE: '售价{price, number, CNY}',
  PHOTO: '你有{num}张照片',
};
