@import '~styles/variables';

// .left {}

.item {
  padding: 12px 24px;
  cursor: pointer;

  &:hover {
    background-color: @products-drawer-hover-background-1;
  }
}

.item-label {
  display: block;
  width: 100%;
  color: @products-title-color;
  font-size: 13px;

  &:hover {
    color: @products-title-color;
  }
}
