@import '~styles/variables';

.sl-simple-table {
  :global {
    .ant-table-content {
      .ant-table-thead {
        tr {
          .ant-table-cell {
            padding: 8px 8px 8px 0;
          }
        }
      }
    }

    .ant-table-column-sorters {
      padding: 8px 0;
    }

    .ant-table-tbody > tr > td {
      padding: 8px 8px 8px 0;
      word-break: break-all;
    }
  }
}
