@import '~styles/variables';

.register {
  float: right;
}

.welcome {
  margin-top: 24px;
  margin-bottom: 24px;
}

.login-form {
  width: 310px;
}

.login-error {
  padding: 8px 12px;
  color: @login-error;
  font-size: 14px;
  line-height: 1.5;
  background: #f2dede;
  border: 1px solid @login-error;
  border-radius: 4px;

  :global {
    .anticon {
      margin-right: 8px;
    }
  }
}
