export const SETTING_DESC = {
  flavor_families: t(
    'Flavor families, used to configure the instance flavor classification'
  ),
  gpu_models: t(
    'GPU model, used when configuring Compute Optimized Type with GPU'
  ),
  usb_models: t('USB model, used when configuring instance flavor'),
};
