.items {
  float: left;
  width: 100%;
}

.item {
  position: relative;
  float: left;
  width: 100%;
  margin-top: 8px;

  :global {
    .ant-form-item-has-success
      .ant-select:not(.ant-select-borderless)
      .ant-select-selector {
      border: 1px solid #d9d9d9 !important;
    }
  }
}

.item-detail {
  width: 90%;
}

.float {
  float: left;
}

.remove-btn,
.add-btn {
  padding-right: 8px;
  padding-left: 0;
}

.desc {
  margin-left: 16px;
}
