.detail {
  margin: 0 16px;
  padding: 24px;
  background-color: #fff;
  border: 1px solid #efefef;

  /* stylelint-disable-next-line selector-class-pattern */
  .public_key {
    text-overflow: ellipsis;
    -ms-word-wrap: break-word;
    word-wrap: break-word;
    -ms-word-break: break-all;
    word-break: break-all;
  }
}
