// Copyright 2021 99cloud
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

export const combosBackground = [
  'rgba(0,104,255,0.05)',
  'rgba(94,246,255,0.08)',
  'rgba(236,220,255,0.08)',
  'rgba(255,255,240,0.3)',
];

export const combosColors = ['#9AC3FF', '#A5FFF0', '#ECDCFF', '#FFFF00'];

export const subnetsColors = [
  ['#9AC3FF', '#4C95FF', '#0068FF', '#0080FF', '#0066CC', '#0000FF'],
  ['#CCF2EC', '#92DACF', '#46B1A5', '#CCF2EC', '#92DACF', '#46B1A5'],
  ['#ECDCFF', '#D6B3FF', '#BA80FF', '#ECDCFF', '#D6B3FF', '#BA80FF'],
  ['#FFFACD', '#FFFF00', '#FFD700', '#FFFACD', '#FFFF00', '#FFD700'],
];
