.node-tooltips {
  :global {
    .ant-card-body {
      padding: 15px;
    }
  }

  position: absolute;
}

.image-icon {
  width: 15px;
  height: 15px;
  margin-right: 2px;
}

.descriptions-item {
  :global {
    .ant-descriptions-item-label {
      width: 80px;
    }

    .ant-descriptions-row > th,
    .ant-descriptions-row > td {
      padding-bottom: 8px;
    }
  }
}

.node-card-title {
  font-size: 16px;
}

.card-button {
  :global {
    .ant-btn-primary {
      margin-right: 6px;
    }
  }
}

.main {
  height: 100%;

  :global {
    .ant-btn-primary {
      margin-right: 16px;
    }
  }
}

.container {
  height: calc(100vh - 170px);
  overflow-y: auto;
}
