// Copyright 2021 99cloud
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

module.exports = {
  'primary-color': '#0068FF',
  'link-color': '#0068FF',
  // 'link-hover-color': '#005ADE',
  // 'link-active-color': '005ADE',
  'success-color': '#57E39B',
  'warning-color': '#979797',
  'error-color': '#EB354D',
  'btn-default-color': '#0068FF',
  'btn-default-border': '#0068FF',
  'border-radius-base': '4px',
  'font-size-base': '12px',
};
