.item {
  margin-bottom: 10px;
  background-color: #fff;

  :global {
    .ant-descriptions-item-label {
      width: 100px;
      color: #8a8a8a;
    }

    .ant-descriptions-item-content {
      word-break: break-all;
    }
  }
}
