@import '~styles/variables';

.mb16 {
  margin-bottom: 16px;
}

.tip {
  color: @color-text-body;
}

.tip-title {
  color: @color-text-title;
  font-weight: 400;
}

.tip-icon {
  margin-right: 8px;
  color: rgba(254, 223, 64, 100%);
}

.tip-content {
  margin-left: 20px;
}
