.link-header {
  margin-bottom: 8px;
  padding-left: 8px;
  color: rgba(0, 0, 0, 85%);
  line-height: 44px;
  background-color: white;
}

.link-title {
  margin-right: 8px;
}

.item-next {
  margin-right: 8px;
  margin-left: 8px;
}
