.instance-volume {
  display: block;
  margin-bottom: 8px;
}

.label {
  max-width: 20%;
  margin-right: 10px;
}

.select {
  max-width: 80%;
}

.size-label {
  margin-right: 40px;
  margin-left: 10px;
}
