export const statusMap = {
  500: t('Internal Server Error (code: 500) '),
  501: t('Not Implemented (code: 501) '),
  502: t('Bad Gateway (code: 502) '),
  503: t('Service Unavailable (code: 503) '),
  504: t('Gateway Time-out (code: 504) '),
  505: t('HTTP Version not supported (code: 505) '),
};
