.table-action {
  margin-right: 5px;
}

.more-action-btn {
  width: 100%;
  text-align: center;
}

.table-batch-menu {
  :global {
    .ant-dropdown-menu-item {
      padding-right: 0;
      padding-left: 0;
    }
  }
}
