.header {
  color: rgba(0, 0, 0, 85%);
  font-weight: 500;
  font-size: 16px;
  line-height: 22px;
}

.list {
  background-color: #fff;
  box-shadow: 0 0 10px 0 rgba(0, 0, 0, 5%);

  .item {
    height: 76px;

    .title {
      display: flex;
      color: rgba(0, 0, 0, 65%);
      font-weight: 400;
      font-size: 16px;
    }

    .status {
      display: flex;
      color: rgba(0, 0, 0, 65%);
      font-weight: 400;
      font-size: 14px;
    }
  }
}

.container {
  display: flex;
  flex-direction: column;
  height: 100%;

  .content {
    flex-grow: 1;
    overflow: auto;
  }
}
