(window.webpackJsonp=window.webpackJsonp||[]).push([[12],{4005:function(e,a,t){"use strict";var u=t(19),o=t(21);u(a,"__esModule",{value:!0}),a.default=void 0;var d=o(t(413)),n=o(t(4006)),l=e=>(0,d.default)(n.default,e);a.default=l},4006:function(e,a,t){"use strict";var u=t(19),o=t(21);u(a,"__esModule",{value:!0}),a.default=void 0;var d=o(t(1212)),n=o(t(567)),l=[{path:"/management",component:d.default,routes:[{path:"*",component:n.default}]}];a.default=l}}]);